@@ -1,12 +1,12 @@
 package Test::BDD::Cucumber::Executor;
-$Test::BDD::Cucumber::Executor::VERSION = '0.38';
+$Test::BDD::Cucumber::Executor::VERSION = '0.39';
 =head1 NAME
 
 Test::BDD::Cucumber::Executor - Run through Feature and Harness objects
 
 =head1 VERSION
 
-version 0.38
+version 0.39
 
 =head1 DESCRIPTION
 
@@ -22,6 +22,7 @@ use List::MoreUtils qw/pairwise/;
 use Module::Runtime qw/use_module/;
 use Number::Range;
 use Carp qw/croak/;
+our @CARP_NOT;
 
 use Test::Builder;
 
@@ -37,8 +38,7 @@ if ( ( !$ENV{'TEST_BDD_CUCUMBER_NO_TB_WRAP_TEST'} )
         $ref_trace = Devel::FindRef::track($Test::Builder::Test);
     }
 
-    my $message =
-      sprintf( <<'END', $ref_trace );
+    my $message = sprintf( <<'END', $ref_trace );
 !!! HEY YOU !!!
 Test::BDD::Cucumber needs to be able to wrap $Test::Builder::Test in order to
 properly capture testing output. However, something else has already taken a
@@ -58,14 +58,16 @@ END
     croak $message;
 }
 
-$Test::Builder::Test =
-  Test::BDD::Cucumber::TestBuilderDelegator->new( Test::Builder->new() );
+$Test::Builder::Test
+    = Test::BDD::Cucumber::TestBuilderDelegator->new( Test::Builder->new() );
 
 use Test::BDD::Cucumber::StepContext;
 use Test::BDD::Cucumber::Util;
 use Test::BDD::Cucumber::Model::Result;
 use Test::BDD::Cucumber::Errors qw/parse_error_from_line/;
 
+has '_bail_out' => ( is => 'rw', isa => 'Bool', default => 0 );
+
 =head1 METHODS
 
 =head2 steps
@@ -130,8 +132,8 @@ sub execute {
     my $feature_stash = {};
 
     $harness->feature($feature);
-    my @background =
-      ( $feature->background ? ( background => $feature->background ) : () );
+    my @background = (
+        $feature->background ? ( background => $feature->background ) : () );
 
     # Get all scenarios
     my @scenarios = @{ $feature->scenarios() };
@@ -145,8 +147,7 @@ sub execute {
 
         # Execute the scenario itself
         $self->execute_scenario(
-            {
-                @background,
+            {   @background,
                 scenario      => $scenario,
                 feature       => $feature,
                 feature_stash => $feature_stash,
@@ -187,18 +188,19 @@ sub execute_scenario {
     my ( $self, $options ) = @_;
     my ( $feature, $feature_stash, $harness, $outline, $background_obj,
         $incoming_scenario_stash, $incoming_outline_stash )
-      = @$options{
+        = @$options{
         qw/ feature feature_stash harness scenario background scenario_stash
-          outline_stash
-          /
-      };
+            outline_stash
+            /
+        };
 
     my $is_background = $outline->background;
 
     my $harness_start = $is_background ? 'background'      : 'scenario';
     my $harness_stop  = $is_background ? 'background_done' : 'scenario_done';
 
-    my $outline_stash = $incoming_outline_stash || { short_circuit => 0 };
+    my $outline_stash = $incoming_outline_stash || {};
+    $outline_stash->{'short_circuit'} ||= $self->_bail_out;
 
     # Multiply out Scenario Outlines as appropriate
     my @datasets = @{ $outline->data };
@@ -238,18 +240,14 @@ sub execute_scenario {
 
                 # Set up a context
                 my $context = Test::BDD::Cucumber::StepContext->new(
-                    {
-                        %context_defaults, verb => 'before',
-                    }
-                );
+                    { %context_defaults, verb => 'before', } );
 
-                my $result =
-                  $self->dispatch( $context, $before_step,
+                my $result = $self->dispatch( $context, $before_step,
                     $outline_stash->{'short_circuit'} );
 
                 # If it didn't pass, short-circuit the rest
                 unless ( $result->result eq 'passing' ) {
-                    $outline_stash->{'short_circuit'}++;
+                    $outline_stash->{'short_circuit'} = 1;
                 }
             }
         }
@@ -258,8 +256,7 @@ sub execute_scenario {
         # execute_scenario...
         if ($background_obj) {
             $self->execute_scenario(
-                {
-                    is_background  => 1,
+                {   is_background  => 1,
                     scenario       => $background_obj,
                     feature        => $feature,
                     feature_stash  => $feature_stash,
@@ -273,13 +270,12 @@ sub execute_scenario {
         foreach my $step ( @{ $outline->steps } ) {
 
             # Multiply out any placeholders
-            my $text =
-              $self->add_placeholders( $step->text, $dataset, $step->line );
+            my $text = $self->add_placeholders( $step->text, $dataset,
+                $step->line );
 
             # Set up a context
             my $context = Test::BDD::Cucumber::StepContext->new(
-                {
-                    %context_defaults,
+                {   %context_defaults,
 
                     # Data portion
                     columns => $step->columns || [],
@@ -294,8 +290,7 @@ sub execute_scenario {
                 }
             );
 
-            my $result =
-              $self->find_and_dispatch( $context,
+            my $result = $self->find_and_dispatch( $context,
                 $outline_stash->{'short_circuit'} );
 
             # If it didn't pass, short-circuit the rest
@@ -310,10 +305,7 @@ sub execute_scenario {
 
                 # Set up a context
                 my $context = Test::BDD::Cucumber::StepContext->new(
-                    {
-                        %context_defaults, verb => 'after',
-                    }
-                );
+                    { %context_defaults, verb => 'after', } );
 
                 # All After steps should happen, to ensure cleanup
                 my $result = $self->dispatch( $context, $after_step, 0 );
@@ -360,19 +352,19 @@ sub find_and_dispatch {
     # Short-circuit if we need to
     return $self->skip_step( $context, 'pending',
         "Short-circuited from previous tests" )
-      if $short_circuit;
+        if $short_circuit;
 
     # Try and find a matching step
     my $step = first { $context->text =~ $_->[0] }
     @{ $self->{'steps'}->{ $context->verb } || [] },
-      @{ $self->{'steps'}->{'step'} || [] };
+        @{ $self->{'steps'}->{'step'} || [] };
 
     # Deal with the simple case of no-match first of all
     return $self->skip_step( $context, 'undefined',
-            "No matching step definition for: "
-          . $context->verb . ' '
-          . $context->text )
-      unless $step;
+              "No matching step definition for: "
+            . $context->verb . ' '
+            . $context->text )
+        unless $step;
 
     return $self->dispatch( $context, $step, 0 );
 }
@@ -392,12 +384,12 @@ sub dispatch {
 
     return $self->skip_step( $context, 'pending',
         "Short-circuited from previous tests" )
-      if $short_circuit;
+        if $short_circuit;
 
     # Execute the step definition
     my ( $regular_expression, $coderef ) = @$step;
 
-    # Setup what we'll pass to step_done, with out localized Test::Builder stuff
+  # Setup what we'll pass to step_done, with out localized Test::Builder stuff
     my $output    = '';
     my $tb_return = {
         output  => \$output,
@@ -411,7 +403,7 @@ sub dispatch {
 
     # Make a minumum pass
     $tb_return->{'builder'}
-      ->ok( 1, "Starting to execute step: " . $context->text );
+        ->ok( 1, "Starting to execute step: " . $context->text );
 
     # Say we're about to start it up
     $context->harness->step($context);
@@ -425,34 +417,39 @@ sub dispatch {
         # Localize test builder
         local $Test::Builder::Test->{'_wraps'} = $tb_return->{'builder'};
 
-        # Execute!
-        eval {
-            no warnings 'redefine';
-
-            # Set S and C to be step-specific values before executing the step
-            local *Test::BDD::Cucumber::StepFile::S = sub {
-                return $context->stash->{'scenario'};
-            };
-            local *Test::BDD::Cucumber::StepFile::C = sub {
-                return $context;
-            };
-
-            # Take a copy of this. Turns out actually matching against it
-            # directly causes all sorts of weird-ass eisenbugs which mst has
-            # promised to investigate.
-            my $text = $context->text;
-
-            # Save the matches
-            $context->matches( [ $text =~ $regular_expression ] );
-
-            # Save the location of matched subgroups for highlighting hijinks
-            my @starts = @-;
-            my @ends   = @+;
-            @match_locations = pairwise { [ $a, $b ] } @starts, @ends;
-
-            # OK, actually execute
-            $coderef->($context);
+        no warnings 'redefine';
+        local *Test::Builder::BAIL_OUT = sub {
+            my ( $tb, $message ) = @_;
+            $self->_bail_out(1);
+            local @CARP_NOT = qw(Test::More Test::BDD::Cucumber::Executor);
+            croak("BAIL_OUT() called: $message");
         };
+
+        # Execute!
+
+        # Set S and C to be step-specific values before executing the step
+        local *Test::BDD::Cucumber::StepFile::S = sub {
+            return $context->stash->{'scenario'};
+        };
+        local *Test::BDD::Cucumber::StepFile::C = sub {
+            return $context;
+        };
+
+        # Take a copy of this. Turns out actually matching against it
+        # directly causes all sorts of weird-ass eisenbugs which mst has
+        # promised to investigate.
+        my $text = $context->text;
+
+        # Save the matches
+        $context->matches( [ $text =~ $regular_expression ] );
+
+        # Save the location of matched subgroups for highlighting hijinks
+        my @starts = @-;
+        my @ends   = @+;
+        @match_locations = pairwise { [ $a, $b ] } @starts, @ends;
+
+        # OK, actually execute
+        eval { $coderef->($context) };
         if ($@) {
             $Test::Builder::Test->ok( 0, "Test compiled" );
             $Test::Builder::Test->diag($@);
@@ -465,16 +462,14 @@ sub dispatch {
 
         # Create the result object
         $result = Test::BDD::Cucumber::Model::Result->new(
-            {
-                result => $status,
+            {   result => $status,
                 output => $output
             }
         );
-
     }
 
-    my @clean_matches =
-      $self->_extract_match_strings( $context->text, \@match_locations );
+    my @clean_matches
+        = $self->_extract_match_strings( $context->text, \@match_locations );
     @clean_matches = [ 0, $context->text ] unless @clean_matches;
 
     # Say the step is done, and return the result. Happens outside
@@ -490,12 +485,12 @@ sub _extract_match_strings {
 
     # Clean up the match locations
     my @match_locations = grep {
-        ( $_->[0] != $_->[1] ) &&    # No zero-length matches
-                                     # And nothing that matched the full string
-          ( !( ( $_->[0] == 0 ) && ( ( $_->[1] == length $text ) ) ) )
-      } grep {
+        ( $_->[0] != $_->[1] ) &&   # No zero-length matches
+                                    # And nothing that matched the full string
+            ( !( ( $_->[0] == 0 ) && ( ( $_->[1] == length $text ) ) ) )
+        } grep {
         defined $_ && ref $_ && defined $_->[0] && defined $_->[1]
-      } @$locations;
+        } @$locations;
 
     return unless @match_locations;
 
@@ -507,7 +502,7 @@ sub _extract_match_strings {
         # expected for nested matches
         no warnings;
         $range->addrange( $_->[0] . '..' . ( $_->[1] - 1 ) )
-          for @match_locations;
+            for @match_locations;
     }
 
     # Walk the string, splitting
@@ -530,16 +525,16 @@ sub _test_status {
     my $self    = shift;
     my $builder = shift;
 
-    my $results =
-        $builder->can("history")
-      ? $self->_test_status_from_history($builder)
-      : $self->_test_status_from_details($builder);
+    my $results
+        = $builder->can("history")
+        ? $self->_test_status_from_history($builder)
+        : $self->_test_status_from_details($builder);
 
     # Turn that in to a Result status
     return
-        $results->{'fail'} ? 'failing'
-      : $results->{'todo'} ? 'pending'
-      :                      'passing';
+          $results->{'fail'} ? 'failing'
+        : $results->{'todo'} ? 'pending'
+        :                      'passing';
 }
 
 sub _test_status_from_details {
@@ -591,8 +586,7 @@ sub skip_step {
 
     # Create a result object
     my $result = Test::BDD::Cucumber::Model::Result->new(
-        {
-            result => $type,
+        {   result => $type,
             output => '1..0 # SKIP ' . $reason
         }
     );
